<?php

/**

* 2014 Jorge Vargas

*

* NOTICE OF LICENSE

*

* This source file is subject to the End User License Agreement (EULA)

*

* See attachmente file LICENSE

*

* @author    Jorge Vargas <jorgevargaslarrota@hotmail.com>

* @copyright 2007-2014 Jorge Vargas

* @license   End User License Agreement (EULA)

* @package   sociallogin

* @version   1.0

*/



class OrderOpcController extends OrderOpcControllerCore

{

	/**

	 * Assign template vars related to page content

	 * @see OrderOpcControllerCore::initContent()

	 */

	public function initContent()

	{

		parent::initContent();



		/* id_carrier is not defined in database before choosing a carrier, set it to a default one to match a potential cart _rule */

		if (empty($this->context->cart->id_carrier))

		{

			$checked = $this->context->cart->simulateCarrierSelectedOutput();

			$checked = ((int)Cart::desintifier($checked));

			$this->context->cart->id_carrier = $checked;

			$this->context->cart->update();

			CartRule::autoRemoveFromCart($this->context);

			CartRule::autoAddToCart($this->context);

		}



		// SHOPPING CART

		$this->_assignSummaryInformations();

		// WRAPPING AND TOS

		$this->_assignWrappingAndTOS();



		if (Configuration::get('PS_RESTRICT_DELIVERED_COUNTRIES'))

			$countries = Carrier::getDeliveredCountries($this->context->language->id, true, true);

		else

			$countries = Country::getCountries($this->context->language->id, true);



		// If a rule offer free-shipping, force hidding shipping prices

		$free_shipping = false;

		foreach ($this->context->cart->getCartRules() as $rule)

			if ($rule['free_shipping'] && !$rule['carrier_restriction'])

			{

				$free_shipping = true;

				break;

			}



		$this->context->smarty->assign(array(

			'free_shipping' => $free_shipping,

			'isGuest' => isset($this->context->cookie->is_guest) ? $this->context->cookie->is_guest : 0,

			'countries' => $countries,

			'sl_country' => (int)Tools::getCountry(),

			'PS_GUEST_CHECKOUT_ENABLED' => Configuration::get('PS_GUEST_CHECKOUT_ENABLED'),

			'errorCarrier' => Tools::displayError('You must choose a carrier.', false),

			'errorTOS' => Tools::displayError('You must accept the Terms of Service.', false),

			'isPaymentStep' => (bool)(isset($_GET['isPaymentStep']) && $_GET['isPaymentStep']),

			'genders' => Gender::getGenders(),

			'one_phone_at_least' => (int)Configuration::get('PS_ONE_PHONE_AT_LEAST'),

			'HOOK_CREATE_ACCOUNT_FORM' => Hook::exec('displayCustomerAccountForm'),

			'HOOK_CREATE_ACCOUNT_TOP' => Hook::exec('displayCustomerAccountFormTop')

		));

		$years = Tools::dateYears();

		$months = Tools::dateMonths();

		$days = Tools::dateDays();

		$this->context->smarty->assign(array(

			'years' => $years,

			'months' => $months,

			'days' => $days,

		));



		/* Load guest informations */

		if ($this->isLogged && $this->context->cookie->is_guest)

			$this->context->smarty->assign('guestInformations', $this->_getGuestInformations());

		// ADDRESS

		if ($this->isLogged)

			$this->_assignAddress();

		// CARRIER

		$this->_assignCarrier();

		// PAYMENT

		$this->_assignPayment();

		Tools::safePostVars();



		$blocknewsletter = Module::getInstanceByName('blocknewsletter');

		$this->context->smarty->assign('newsletter', (bool)($blocknewsletter && $blocknewsletter->active));

		$this->context->smarty->assign('field_required', $this->context->customer->validateFieldsRequiredDatabase());



		$this->_processAddressFormat();



		$this->module = Module::getInstanceByName('sociallogin');

		if ($this->module->active && Configuration::get($this->module->name.'_OVERRIDE'))

			$this->setTemplate($this->module->getTemplatePath('order-opc.tpl'));

		else

			$this->setTemplate(_PS_THEME_DIR_.'order-opc.tpl');

	}

}